/*
 * UbuntuOne Nautilus plugin
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * Copyright 2009-2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <glib/gi18n.h>
#include <libsyncdaemon/syncdaemon-shares-interface.h>
#include "ubuntuone-nautilus.h"

/* this is the wrong place for this, but it'll have to do for now */
gboolean
ubuntuone_nautilus_check_shares_and_public_files (UbuntuOneNautilus *uon, SyncdaemonFolderInfo *folder_info, GtkWidget *widget)
{
	SyncdaemonInterface *interface;
	GHashTable *shares = NULL;
	gboolean result = TRUE;

	interface = syncdaemon_daemon_get_shares_interface (uon->syncdaemon);
	if (SYNCDAEMON_IS_SHARES_INTERFACE (interface)) {
		GSList *shared_list, *l;

		shared_list = syncdaemon_shares_interface_get_shared (SYNCDAEMON_SHARES_INTERFACE (interface));
		for (l = shared_list; l != NULL; l = l->next) {
			SyncdaemonShareInfo *share_info = SYNCDAEMON_SHARE_INFO (shared_list->data);

			if (g_str_has_prefix (syncdaemon_share_info_get_path (share_info),
					      syncdaemon_folder_info_get_path (folder_info))
			    && syncdaemon_share_info_get_accepted (share_info)) {
				gchar *path;
				const gchar *username;
				GSList *users = NULL;

				if (shares == NULL)
					shares = g_hash_table_new (g_str_hash, g_str_equal);

				g_hash_table_lookup_extended (shares,
							      syncdaemon_share_info_get_path (share_info),
							      (gpointer *) &path,
							      (gpointer *) &users);

				/* Add the share and the user it's shared with to the hash table */
				if (syncdaemon_share_info_get_user_visible_name (share_info) != NULL)
					username = syncdaemon_share_info_get_user_visible_name (share_info);
				else
					username = syncdaemon_share_info_get_username (share_info);

				users = g_slist_append (users, (gpointer) username);
				g_hash_table_insert (shares,
						     (gpointer) syncdaemon_share_info_get_path (share_info),
						     (gpointer) users);
			}
		}

		g_slist_free (shared_list);

		/* If there are shares, ask the user what to do */
		if (shares != NULL) {
			GtkWidget *dialog;
			GHashTableIter iter;
			GString *question;
			gchar *path;
			GSList *users;

			question = g_string_new (_("This folder contains folders that have been shared:\n\n"));

			g_hash_table_iter_init (&iter, shares);
			while (g_hash_table_iter_next (&iter, (gpointer) &path, (gpointer) &users)) {
				gint count = 0;

				question = g_string_append (question, "\t- ");
				question = g_string_append (question, path);
				question = g_string_append (question, " (");

				while (users != NULL) {
					gchar *username = (gchar *) users->data;

					if (count > 0)
						question = g_string_append (question, ", ");
					question = g_string_append (question, username);

					users = g_slist_remove (users, username);
					count += 1;
				}

				question = g_string_append (question, ")\n");
			}

			question = g_string_append (question, _("\nThis action will un-share these items. Other users"
								" will no longer be able to access these files."
								" Would you like to proceed?"));

			dialog = gtk_message_dialog_new (
				GTK_WINDOW (gtk_widget_get_toplevel (widget)),
				0, GTK_MESSAGE_QUESTION,
				GTK_BUTTONS_YES_NO,
				"%s", question->str);
			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
				result = FALSE;

			gtk_widget_destroy (dialog);
			g_string_free (question, TRUE);
			g_hash_table_destroy (shares);

			if (!result)
				return FALSE;
		}

	}

	/* Now check for published files */
	if (uon->gotpubs) {
		GHashTableIter iter;
		gchar *key, *value;
		GString *question;
		gboolean has_published = FALSE;

		question = g_string_new (_("This folder contains files that have been published:\n\n"));

		g_hash_table_iter_init (&iter, uon->public);
		while (g_hash_table_iter_next (&iter, (gpointer *) &key, (gpointer *) &value)) {
			if (g_str_has_prefix (key, syncdaemon_folder_info_get_path (folder_info))) {
				has_published = TRUE;

				question = g_string_append (question, "\t- ");
				question = g_string_append (question, key);
				question = g_string_append (question, " (");
				question = g_string_append (question, value);
				question = g_string_append (question, ")\n");
			}
		}

		question = g_string_append (question, _("\nThis action will un-publish these items. Other users"
							" will no longer be able to access these files."
							" Would you like to proceeed?"));

		if (has_published) {
			GtkWidget *dialog;

			dialog = gtk_message_dialog_new (
				GTK_WINDOW (gtk_widget_get_toplevel (widget)),
				0, GTK_MESSAGE_QUESTION,
				GTK_BUTTONS_YES_NO,
				"%s", question->str);
			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
				result = FALSE;

			gtk_widget_destroy (dialog);
		}

		g_string_free (question, TRUE);
	}

	return result;
}
